#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <time.h>

#include "functions.h"

/* ----------------------- *
 * Replacement for strncpy *
 * ----------------------- */

void string_copy(char *dest,char *source,int n)
{
 int i=0;

 while (i<n)
 {
  *(dest+i)=*(source+i);
  i++;
 }

 *(dest+i)='\0';

}



/* ----------------------------------------------------- * 
 * Function extracting IDs from line retrieved by netMod * 
 * Returns length of calling number                      * 
 * ----------------------------------------------------- */

int proccess_line(char *s,char *from,char *to)
{
 char *sep1;
 char *sep2;

 /* Get first string - Called number (MSN) */
 
 sep1=index(s,';');

 /* Return error, if no ; found in string.                              *
  * This prevents multiple pop-up windows, because netMod keeps sending *
  * RING responses, after the fisrt RING calledid;callerid response     */
 
 if (sep1==NULL) return -1;

 string_copy(to,s+5,7);

 /* Get second string - Calling number */
 
 sep2=index(s,'\r');
 string_copy(from,sep1+1,sep2-sep1-1);
 
 return sep2-sep1-1;
}



/* ------------------------------------------------------------ *
 * Function for retrieving a single line (response) from netMod *
 * Returns length of line                                       * 
 * ------------------------------------------------------------ */

int readline(char *s, int fd)
{
 int i=0;
 char c;

 do
 {
  read(fd,&c,1);
  *(s+i)=c;
  i++;
 } while (c!='\n');

 return i;
}


/* ---------------------------------------------------------------- *
 * Place new log entry. Caller ID, called ID, and time call occured *
 * ---------------------------------------------------------------- */

void new_log_entry(int log_fd, char * from, char * to)
{
 time_t now=time(0);
 char entry[80];

 sprintf(entry,
         "%s\tcalled (MSN number %s) on\t%s",
         from,
         to,
         ctime(&now));

 write(log_fd,&entry,strlen(entry));
}


/* --------------------------------------------------- *
 * Initialize netMod device input/output/control modes *
 * Returns 1 on error, or else 0                       *
 * --------------------------------------------------- */
 
int init_netMod(int fd)
{

 struct termios setme;

 setme.c_iflag=IGNBRK;
 setme.c_oflag=NL0|CR0|TAB0|BS0|FF0;
 setme.c_cflag=CREAD|CLOCAL|B115200|CS8|CRTSCTS;
 setme.c_lflag=0;

 if (tcsetattr(fd,TCSANOW,&setme)<0)
 {
  return 1;
 }

 return 0;
}



/* Open Log file in write mode. If it doesn't exist creat it. If it exists *
 * open it in append mode.                                                 *
 * Returns file descriptor of log file, or -1 on error                     */

int open_log()
{
 int logfd=0;
 
 time_t now=time(0);
 char message[80];

 logfd=open(strcat(getenv("HOME"),"/.cid-log"),
            O_WRONLY|O_CREAT|O_APPEND,
	    0644);
 
 if (logfd<0)
 {
  return -1;
 }

 if ( fchown(logfd,getuid(),getgid())<0 )
 {
  return -1;
 }
 
 sprintf(message,"\nCID programm started at %s\n",ctime(&now));
 write(logfd,&message,strlen(message));

 return logfd;
}



/* --------------------------------------------------------------------- *
 * Create /var/lock/LCK..(netMod device) to prevent other processes from *
 * accessing netMod                                                      *
 * Returns 1 on error, or else 0                                         *
 * --------------------------------------------------------------------- */

int lock_netMod()
{
 int lock=0;
 char *content;
 
 content=(char *)calloc(15,1);
	 
 lock=open(lockfile,O_WRONLY | O_CREAT | O_EXCL,0644);

 if (lock<0)
 {
  return 1;
 }

 sprintf(content,"%d %s root\n",getpid(),"Cid_Base");
 write(lock,content,strlen(content)+1);

 close(lock);

 return 0;
 
}



/* ----------------------------------------------------------------- *
 * Set voice transparency mode in netMod (S82 register = 76) so that *
 * incoming calls produce RING responses                             *
 * Returns 1 on error, or else 0                                     *
 * ----------------------------------------------------------------- */
 
int adjust_netMod(int fd,char *line)
{

 write(fd,"ATS82=76\r",9);
 readline(line,fd);
 readline(line,fd);
 
 if ( (line[0]!='O') && (line[1]!='K') )
 {
  return 1;
 }

 return 0;
 
}



/* ----------------------------- *
 * Reset netMod (send ATZ)       *
 * Returns 1 on error, or else 0 *
 * ----------------------------- */
   
int reset_netMod(int fd, char *line)
{

 write(fd,"ATZ\r",4);
 readline(line,fd);
 readline(line,fd);

 if ( (line[0]!='O') && (line[1]!='K') )
 {
  return 1;
 }

 return 0;
 
}



/* ----------------------------------------------------- *
 * End process. Invoked on SIGTERM signal.               *
 * Reset netMod, delete lock file, close file descriptor *
 * ----------------------------------------------------- */
 
void telos()
{
 extern int fd;
 char s[80];

 reset_netMod(fd,s);
 
 close(fd);
 
 unlink(lockfile);
 
 exit(0);

 signal(SIGTERM,telos);
}
